# future.moves package
from __future__ import absolute_import
import sys
__future_module__ = True
from ...future.standard_library import import_top_level_modules

if sys.version_info[0] >= 3:
    import_top_level_modules()
